# -*- coding: utf-8 -*-
import datetime

from kinder.core.dict.models import UnitKind
from gisdo.queue.queue_conf import GisdoContainer
from gisdo.queue.runtime_filters import DateYearFilter
from gisdo.queue.context import Context
from gisdo.utils import AgeDeltas


def get_queue_index(
        index_number, unit, age_range=None, on_date=datetime.date.today,
        check_unit=True):
    """Вычисление новых показателей по очереди"""

    qs = get_queue_index_collection(
        index_number, unit, age_range=age_range, on_date=on_date,
        check_unit=check_unit)

    return qs.count()


def get_queue_index_collection(
        index_number, unit, age_range=None, on_date=datetime.date.today,
        check_unit=True):
    """Вычисление новых показателей по очереди"""

    assert 'p{0}'.format(
        index_number.replace('.', '_')
    ) in GisdoContainer.index_filters, "Queue config failed"

    # Формирование контекста на основе показателей.
    ctx = Context(unit, on_date=on_date, first_dou_only=True)
    if age_range is not None:
        ctx = ctx.add_filters(DateYearFilter(age_range))

    # Получение билдера
    cont = GisdoContainer()
    builder = cont.get_builder(index_number)

    # Построение QuerySet-а
    qs = builder.build(ctx)
    if check_unit:
        qs = qs.filter(
            declarationunit__unit_id=unit.id,
            declarationunit__ord=1,
            declarationunit__unit__kind_id=UnitKind.DOU)

    return qs


def get_count(collection, age_filter):
    u"""Определение количества записей в коллекции подходящих под фильтр.

    Функция применяет фильтр age_filter к коллекции collection и возвращает
    количество отфильтрованных записей.
    """

    return len([
        declaration
        for declaration in collection
        if age_filter(declaration)
    ])


def get_age_filter(age, on_date):
    u"""Функция строит фильтр возраста."""

    down, up = AgeDeltas.get_category_deltas(age, on_date)

    filter_ = lambda rec: rec["children__date_of_birth"] > down
    if up:
        if age == u'от 2 месяцев до 6 месяцев':
            filter_ = lambda rec: (down < rec["children__date_of_birth"])
        else:
            filter_ = lambda rec: (
                down < rec["children__date_of_birth"] <= up
            )

    return filter_
